//
//  ImagePuzzleViewController.m
//  ImagePuzzle
//
//  Created by 유동근 on 09. 05. 28.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//

#import "ImagePuzzleViewController.h"
#import "BorderView.h"

@implementation ImagePuzzleViewController

@synthesize baseImage;

/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
	[super loadView];
	[self mixImage:[UIImage imageNamed:@"sample.png"]];
	[[UIAccelerometer sharedAccelerometer] setDelegate:self];
}

-(void) mixImage:(UIImage*) image {
	for (UIView* view in [self.view subviews])
		[view removeFromSuperview];
	
	self.baseImage = image;
	UIImageView* lastImageView;
	for (int r = 0; r < MAX_ROW ;r++)
		for (int c = 0; c < MAX_COL ;c++)
		{
			UIImageView* imageView = [[UIImageView alloc] init];
			UIView* borderView = [[BorderView alloc] initWithFrame:CGRectMake(0, 0, IMAGE_WIDTH, IMAGE_WIDTH)];
			[imageView addSubview:borderView];
			[borderView release];
			
			CGRect rcImage = CGRectMake(c*IMAGE_WIDTH, r*IMAGE_WIDTH, IMAGE_WIDTH, IMAGE_WIDTH);
			UIImage* cropImage = [UIImage imageWithCGImage:
								  CGImageCreateWithImageInRect([baseImage CGImage], rcImage)];
			[imageView setImage:cropImage];
			imageView.tag = r*c;	// 순번태그(퍼즐이 모두 정렬되었는지 판단)
			[self.view addSubview:imageView];
			[imageView release];
			lastImageView = imageView;
		}
	[lastImageView setAlpha:0.1f];
	
	NSArray* subViews = [self.view subviews];
	int cnt = MAX_COL*MAX_ROW;
	int b[MAX_COL*MAX_ROW];
	for (int i = 0; i < cnt ;i++)
		b[i] = i;
	
	srand(time (NULL));
	for (int i = 0; i < cnt ;i++)
	{
		int rnd = rand()%(cnt-i);
		imageViews[i] = [subViews objectAtIndex:b[rnd]];
		b[rnd] = b[cnt-i-1];
		CGRect frame = CGRectMake((i%MAX_COL)*IMAGE_WIDTH, (i/MAX_COL)*IMAGE_WIDTH, 
								  IMAGE_WIDTH, IMAGE_WIDTH);
		[imageViews[i] setFrame:frame];
		if (imageViews[i] == lastImageView)
			iEmptyImage = i;
	}
}

/*
// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
}
*/

- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event {
	UITouch*	touch = [touches anyObject];
	CGPoint		location = [touch locationInView:self.view];
	int sc = location.x/IMAGE_WIDTH;
	int sr = location.y/IMAGE_WIDTH;
	int iSelectImage = sr*MAX_COL + sc;
	int r = iEmptyImage/MAX_COL;
	int c = iEmptyImage%MAX_COL;
	
	if (iSelectImage == iEmptyImage) {
		UIImagePickerController* imagePicker = [[UIImagePickerController alloc] init];
		if ([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera])
			imagePicker.sourceType = UIImagePickerControllerSourceTypeCamera;
		else
			imagePicker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
		imagePicker.title = @"사진 직기";
		imagePicker.allowsImageEditing = YES;
		imagePicker.delegate = self;
		[self presentModalViewController:imagePicker animated:YES];
		[imagePicker release];
		return;
	}
	
	if ((sr-1 == r && sc == c) ||
		(sr == r && sc-1 == c) ||
		(sr == r && sc+1 == c) ||
		(sr+1 == r && sc == c)) {
		UIImageView* imageViewSelect = imageViews[iSelectImage];
		UIImageView* imageViewEmpty  = imageViews[iEmptyImage];
		
		CGContextRef context = UIGraphicsGetCurrentContext(); 
		[UIView beginAnimations:nil context:context];
		[UIView setAnimationCurve:UIViewAnimationCurveLinear];
		[UIView setAnimationDuration:0.2]; 
		
		CGRect rcSelect = imageViewSelect.frame;
		CGRect rcEmpty  = imageViewEmpty.frame;
		[imageViewSelect setFrame:rcEmpty];
		[imageViewEmpty setFrame:rcSelect];
		imageViews[iSelectImage] = imageViewEmpty;
		imageViews[iEmptyImage] = imageViewSelect;
		iEmptyImage = iSelectImage;
		
		[UIView commitAnimations]; 
		
		BOOL bArrange = YES;
		for (int i = 0; i < MAX_COL*MAX_ROW ;i++) {
			if (i != imageViews[i].tag)
				bArrange = NO;
		}
		
		for (int i = 0; i < MAX_COL*MAX_ROW ;i++) {
			BorderView* borderView = (BorderView*)[imageViews[i].subviews objectAtIndex:0];
			[borderView setHideBorder:bArrange];
		}
	}	
}

//--------------------------------------------------------------------------------------------------
- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingImage:(UIImage *)image editingInfo:(NSDictionary *)editingInfo
{
	CGSize size = CGSizeMake(SCREEN_WIDTH, SCREEN_HEIGHT);
	UIGraphicsBeginImageContext(size); 
	[image drawInRect:CGRectMake(0,0,size.width,size.height)];
	UIImage* newImage = UIGraphicsGetImageFromCurrentImageContext(); 
	UIGraphicsEndImageContext(); 
	
	[self mixImage:newImage];
	[self imagePickerControllerDidCancel:picker];
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker
{
	[picker dismissModalViewControllerAnimated:YES];
}

- (void)accelerometer:(UIAccelerometer *)accelerometer 
		didAccelerate:(UIAcceleration *)acceleration {
	static int mix = 0;
	float x = -[acceleration x];
	float y = [acceleration y];
	float angle = atan2 (y, x);
	if (mix == 0 && 1 < angle) {
		[self mixImage:self.baseImage];
		mix = 1;
	}
	else if (angle < 0) 
		mix = 0;

	NSLog(@"%f, %f, %f, %f", [acceleration x], [acceleration y], [acceleration z], angle);
}
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return NO;
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
    // Release anything that's not essential, such as cached data
}


- (void)dealloc {
    [super dealloc];
	[baseImage release];
}

@end
